/* pmudmon applet
 ** Copyright (C) 2000 Mike Palczewski mpalczew@u.washington.edu
 ** 
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; either version 2
 ** of the License, or (at your option) any later version.
 ** 
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 ** 
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 ** 02111-1307, USA.
 **/
#include <pthread.h>
#include "main.h"
#include "properties.h"
#include "conf.h"

char buffer[80];
int sockfd;

int menu(GtkWidget *widget, GdkEventButton *event);
int on_delete(GtkWidget* window,GdkEvent *event,gpointer data);
void doabout (AppletWidget*,gpointer);
void getLine();
gint update_display(gpointer data,gint source, GdkInputCondition condition);

int thread_write[2],thread_read[2];

void initpmud();
void setup_threads();

int main(int argc, char **argv)
	{
#ifdef NO_APPLET		
	GtkWidget* event_box;
#endif
	GtkWidget* applet;
	GtkWidget* box;
	conf* config;
	int i=1;
	
	initpmud();
	setup_threads();	
#ifndef NO_APPLET
	applet_widget_init(PACKAGE, VERSION, argc, argv,
			        NULL, 0, NULL);
	applet = applet_widget_new(PACKAGE);
#else
	gnome_init(PACKAGE, VERSION, argc, argv);
	applet=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_wmclass(GTK_WINDOW(applet),"pmudmon","pmudmon");
	gtk_window_set_title(GTK_WINDOW(applet),"pmudmon");
#endif
	box = gtk_vbox_new(FALSE,0);
	temperature = gtk_label_new("0 C");
	gdk_input_add(thread_write[0],GDK_INPUT_READ,
			GTK_SIGNAL_FUNC(update_display),NULL);
	
	config=get_conf();
	progress=gtk_progress_bar_new();
	gtk_widget_set_usize(progress,config->width,config->height);
	gtk_progress_set_show_text(GTK_PROGRESS(progress),1);
	gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(progress),config->orientation);	
#ifndef NO_APPLET
	gtk_box_pack_start(GTK_BOX(box), progress, FALSE,FALSE,0);
	gtk_box_pack_end(GTK_BOX(box), temperature, FALSE,FALSE,0);
	applet_widget_add(APPLET_WIDGET(applet), GTK_WIDGET(box)); 
#else
	event_box = gtk_event_box_new ();
	gtk_container_add (GTK_CONTAINER (applet), event_box);
	gtk_widget_set_events (event_box, GDK_BUTTON_PRESS_MASK);
        gtk_signal_connect (GTK_OBJECT(event_box), "button_press_event",
                               GTK_SIGNAL_FUNC (menu), NULL);
	
	gtk_box_pack_start(GTK_BOX(box), progress, FALSE,FALSE,0);
	gtk_box_pack_end(GTK_BOX(box), temperature, FALSE,FALSE,0);
	gtk_container_add (GTK_CONTAINER (event_box),box);

	gtk_widget_show(event_box);
	gtk_widget_realize(event_box);
#endif
	gtk_widget_show(progress);
	gtk_widget_show(temperature);
	gtk_widget_show(applet);
	gtk_widget_show(GTK_WIDGET(box));
	timeout=gtk_timeout_add( config->time*1000,signal_thread,NULL);
	
#ifndef NO_APPLET
	applet_widget_register_stock_callback((AppletWidget*)applet,
			"Properties",GNOME_STOCK_MENU_PROP,"Properties",
			doproperties,NULL);
	applet_widget_register_stock_callback((AppletWidget*)applet,"About...",
			GNOME_STOCK_MENU_ABOUT,"About...",doabout,NULL);
#else
	gtk_signal_connect(GTK_OBJECT(applet),"delete-event",
			GTK_SIGNAL_FUNC(on_delete),NULL);

#endif	

	free_conf(config);
#ifndef NO_APPLET
	applet_widget_gtk_main(); 
#else
	gtk_main();
#endif
	write(thread_read[1],&i,sizeof(int));
	close(sockfd);
	return 0;
}


gint signal_thread(gpointer data) {
	int i=0;
	write(thread_read[1],&i,sizeof(int));
	return TRUE;
}


void initpmud() {
	int len;

	struct sockaddr_in address;
	int result;
	
	sockfd=socket(AF_INET,SOCK_STREAM,0);
	address.sin_family = AF_INET;
	address.sin_addr.s_addr = inet_addr("127.0.0.1");
	address.sin_port=879;
	len=sizeof(address);

	result=connect(sockfd,(struct sockaddr *)&address,len);

	if (result == -1) {
		puts("cannot connect to pmud");
		exit(0);
	}
}

gchar *get_temperature(void) {
	FILE *cpuinfo = NULL;
	char *buf = malloc(256);
	char *tmp = malloc(256);
	
	cpuinfo = fopen("/proc/cpuinfo","r");
	if(cpuinfo == NULL) /* sh*t, can't open file */
		return "";

	/* go to temperature line */
	do {
		fgets(buf, 256, cpuinfo);
	} while( !feof(cpuinfo) && !strstr(buf, "temperature"));

	fclose(cpuinfo);
	if(!strstr(buf, "temperature")) {
		/* temperature line not found :-( */
		return "";
	}
	
	/* get it */
	tmp = strtok(buf, ":");  /* skip "temperature :" */
	tmp = strtok(NULL, ":"); /* get the second part */
	buf = tmp;
	tmp = strtok(buf, "C");  /* get the numbers */
	buf = tmp;
	tmp = strtok(buf, " ");  /* skip first space */
	tmp[strlen(tmp)]='\0';  /* skip last space */
	strcat(tmp,"C");       /* add degrees */
	return tmp;
}

gint update_display(gpointer data,gint source, GdkInputCondition condition ) {
	int stat[3];
	conf* config;
	GtkRcStyle* style;

	read(source,stat,sizeof(stat));
	config=get_conf();
	if (stat[1] != 0 &&  stat[2] != 0)  
		gtk_progress_set_percentage(GTK_PROGRESS(progress),(float)stat[1]/(float)stat[2]);	
	switch (stat[0]) {
	   	case 111:
			gtk_progress_set_format_string(GTK_PROGRESS(progress),config->format_charging);
			style=gtk_rc_style_new();
			style->color_flags[GTK_STATE_PRELIGHT]=GTK_RC_BG;
			style->color_flags[GTK_STATE_NORMAL]=GTK_RC_BG;
			style->bg[GTK_STATE_NORMAL].red=config->bg_red;
			style->bg[GTK_STATE_NORMAL].green=config->bg_green;
			style->bg[GTK_STATE_NORMAL].blue=config->bg_blue;
			style->bg[GTK_STATE_PRELIGHT].red=config->ca_red;
			style->bg[GTK_STATE_PRELIGHT].green=config->ca_green;
			style->bg[GTK_STATE_PRELIGHT].blue=config->ca_blue;
			gtk_widget_modify_style(progress,style);
			gtk_rc_style_unref(style);

			break;
		case 001:
			gtk_progress_set_format_string(GTK_PROGRESS(progress),config->format_battery);
			style=gtk_rc_style_new();
			style->color_flags[GTK_STATE_PRELIGHT]=GTK_RC_BG;
			style->color_flags[GTK_STATE_NORMAL]=GTK_RC_BG;
			style->bg[GTK_STATE_NORMAL].red=config->bg_red;
			style->bg[GTK_STATE_NORMAL].green=config->bg_green;
			style->bg[GTK_STATE_NORMAL].blue=config->bg_blue;
			style->bg[GTK_STATE_PRELIGHT].red=config->bt_red;
			style->bg[GTK_STATE_PRELIGHT].green=config->bt_green;
			style->bg[GTK_STATE_PRELIGHT].blue=config->bt_blue;
			gtk_widget_modify_style(progress,style);
			gtk_rc_style_unref(style);
			break;
		case 101:
			gtk_progress_set_format_string(GTK_PROGRESS(progress),config->format_charged);	
			style=gtk_rc_style_new();
			style->color_flags[GTK_STATE_PRELIGHT]=GTK_RC_BG;
			style->color_flags[GTK_STATE_NORMAL]=GTK_RC_BG;
			style->bg[GTK_STATE_NORMAL].red=config->bg_red;
			style->bg[GTK_STATE_NORMAL].green=config->bg_green;
			style->bg[GTK_STATE_NORMAL].blue=config->bg_blue;
			style->bg[GTK_STATE_PRELIGHT].red=config->cd_red;
			style->bg[GTK_STATE_PRELIGHT].green=config->cd_green;
			style->bg[GTK_STATE_PRELIGHT].blue=config->cd_blue;
			gtk_widget_modify_style(progress,style);
			gtk_rc_style_unref(style);
			break;
		case 100:
		  gtk_progress_set_format_string(GTK_PROGRESS(progress),"no battery");//change this
			break;
		default:
			gtk_progress_set_format_string(GTK_PROGRESS(progress),"Unknown condition %p%%");
			break;
	}
	if (stat[0] == 001 && config->shutdown && config->shutdown  > ((float)stat[1]/(float)stat[2] * 100)) system("/sbin/halt");
	
	gtk_label_set_text(GTK_LABEL(temperature), get_temperature());
	
	free_conf(config);
	return TRUE;
}

void getLine() {
    char b2[80];
	char *b3;

	write (sockfd,"\n",1);
	read(sockfd,b2,80);

	b3=strtok(b2,"\n");	
	strcpy(buffer,b3);
}	


void doabout(AppletWidget* widget,gpointer data) {

	gchar *authors[] = {
	    	"mpalzcew@u.washington.edu",
	        NULL
			   };
 	
	gtk_widget_show (gnome_about_new (PACKAGE, VERSION,
        	"Copyright 2000",(const gchar **) authors,
    		"Battery meter for the ibook ",NULL));

}

void run_thread() {
	int a,stat[3];	
	int quit=0;
	getLine();

	while (quit == 0) {
		getLine();
		a=sscanf(buffer,"%*s {%d %d %d %*s %*s %*s",stat,stat+1,stat+2);
		write(thread_write[1],&stat,sizeof(stat));
		read(thread_read[0],&quit,sizeof(int));
	}
}


void setup_threads() {
	pthread_t thread;
	
	pipe(thread_read);
	pipe(thread_write);

	
	pthread_create(&thread,NULL,(void*)(void*)run_thread,NULL);

}


#ifdef NO_APPLET
int on_delete(GtkWidget* window,GdkEvent *event,gpointer data) {
	gtk_main_quit();
	return FALSE;
}


int menu(GtkWidget *widget, GdkEventButton *event) {
	GtkWidget* menuitem;
	GtkWidget* menu;
	
	if (event->button != 3 )
		return FALSE;
	

	menu=gtk_menu_new();
	menuitem=gnome_stock_menu_item(GNOME_STOCK_MENU_PROP,"Properties");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU(menu),menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem),"activate",GTK_SIGNAL_FUNC(doproperties),NULL);
			
	menuitem=gnome_stock_menu_item(GNOME_STOCK_MENU_ABOUT,"About...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU(menu),menuitem);
	
	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,3,time(NULL));
	return TRUE;
}
#endif
